<?php
/**
 * Internal Links
 */

wp_enqueue_script( 'wpaa-chartjs' );
wp_enqueue_script( 'wpaa-charts' );

$wpaa_internal_threshold = (int) get_option( 'wpaa_internal_threshold', 4 );

// Get a list of all WordPress posts, pages and their links
$args = [
    'post_type'      => [
        'post',
        'page',
    ],
    'posts_per_page' => -1,
];

$pages = get_posts( $args );

$link_data = [];

foreach ( $pages as $page ) {
    $link_data[ $page->ID ] = [
        'title' => $page->post_title,
        'url'   => get_permalink( $page->ID ),
        'links' => [],
    ];

    // Parse the content to find links
    preg_match_all( '/<a\s+href=["\'](http[s]?:\/\/[^"\']+)["\']/i', $page->post_content, $matches );

    foreach ( $matches[1] as $link ) {
        $link_data[ $page->ID ]['links'][] = $link;
    }
}

// Define a custom comparison function for sorting by the number of links
function wpaa_sort_by_link_count($a, $b) {
    return count( $b['links']) - count( $a['links'] ); // Sort in descending order
}

// Sort the $link_data array by the number of links in descending order
uasort( $link_data, 'wpaa_sort_by_link_count' );

// Calculate link counts
$link_counts = [];

foreach ( $link_data as $page ) {
    foreach ( $page['links'] as $link ) {
        if ( ! isset( $link_counts[ $link ] ) ) {
            $link_counts[ $link ] = 1;
        } else {
            $link_counts[ $link ]++;
        }
    }
}

// Sort pages by link count to create a ranking
arsort( $link_counts );
?>

<style>
.w3p-container {
    display: grid;
    grid-template-columns: repeat(2, 1fr);
    grid-gap: 4em;
}
</style>

<h2>Internal Link Rankings & Page Link Count</h2>

<hr>

<h2>Page Links</h2>

<div class="thin-ui-grid thin-ui-grid--nowrap">
    <div class="thin-ui-col col--inner">
        <table width="100%" cellpadding="8" cellspacing="0" class="page-links-table--all">
            <thead>
                <tr>
                    <th scope="col" style="width:50%">Page</th>
                    <th scope="col" style="width:50%">Links to</th>
                </tr>
            </thead>
            <tbody>

            <?php
            foreach ( $link_data as $page ) {
                echo '<tr>
                    <td><a href="' . $page['url'] . '">' . $page['title'] . '</a></td>
                    <td>';

                    if ( count( $page['links'] ) > 0 ) {
                        echo '<details>
                            <summary><b>' . count( $page['links'] ) . '</b> links found. Click to expand.</summary>';

                            foreach ( $page['links'] as $link ) {
                                echo '<div><a href="' . $link . '">' . $link . '</a></div>';
                            }
                        echo '</details>';
                    } else {
                        echo '<b>No links found!</b>';
                    }

                    echo '</td>
                </tr>';
            }
            ?>

            </tbody>
        </table>
    </div>
</div>

<h2>Link Rankings</h2>

<div class="thin-ui-grid thin-ui-grid--nowrap">
    <div class="thin-ui-col col--inner">
        <div class="w3p-container">
            <div>
                <table width="100%" cellpadding="8" cellspacing="0" class="link-rankings-table--all">
                    <thead>
                        <tr>
                            <th scope="col" style="width:95%">Link</th>
                            <th scope="col" style="width:5%">Count</th>
                        </tr>
                    </thead>
                    <tbody>

                    <?php
                    foreach ( $link_counts as $link => $count ) {
                        if ( $count > $wpaa_internal_threshold ) {
                            echo '<tr>
                                <td>' . $link . '</td>
                                <td>' . $count . '</td>
                            </tr>';
                        }
                    }
                    ?>

                    </tbody>
                </table>
            </div>
            <div>
                <canvas id="linkRankingsChart" width="400" height="400" style="width:100%;"></canvas>
            </div>
        </div>
    </div>
</div>

<?php
$chart_labels = [];
$chart_data   = [];

foreach ( $link_counts as $link => $count ) {
    $chart_labels[] = $link;
    $chart_data[]   = $count;
}
?>
<script>
document.addEventListener("DOMContentLoaded", (event) => {
    var ctx = document.getElementById('linkRankingsChart').getContext('2d');

    // Function to calculate opacity based on link count
    function calculateOpacity(count) {
        // Define the opacity range (adjust as needed)
        var minOpacity = 0.3;
        var maxOpacity = 1.0;
        
        // Calculate opacity based on link count
        var opacity = minOpacity + (count / <?php echo max( $chart_data ); ?>) * (maxOpacity - minOpacity);
        return opacity;
    }

    var chartData = <?php echo json_encode( $chart_data ); ?>;
    var chartLabels = <?php echo json_encode( $chart_labels ); ?>;
    var maxLinkCount = <?php echo max( $chart_data ); ?>;
    var backgroundColors = [];

    // Generate unique colors for each slice
    for (var i = 0; i < chartData.length; i++) {
        var opacity = calculateOpacity(chartData[i]);

        backgroundColors.push(`rgba(88, 0, 255, ${opacity})`);
    }

    // Filter the data to include only slices with values greater than 1
    var filteredData = chartData.filter(function (value) {
        return value > <?php echo $wpaa_internal_threshold; ?>;
    });
    // Create labels for the filtered data
    var filteredLabels = chartLabels.filter(function (_, index) {
        return chartData[index] > 1;
    });

    var chart = new Chart(ctx, {
        type: 'doughnut',
        data: {
            labels: filteredLabels,
            datasets: [{
                data: filteredData,
                backgroundColor: backgroundColors,
                hoverBackgroundColor: 'red', // Set hover color
                hoverBorderWidth: 2, // Set hover border width
                borderWidth: 2, // Set the border width to 0 to remove borders
                borderColor: 'white', // Set the border color to transparent
            }]
        },
        options: {
            responsive: true, // Enable responsiveness
            maintainAspectRatio: true, // Allow the chart to scale both horizontally and vertically
            cutout: '60%', // Adjust the size of the hole in the center of the donut
            plugins: {
                legend: false, // Disable the legend
            },
            tooltips: {
                callbacks: {
                    label: function(context) {
                        return chartLabels[context.dataIndex] + ' (Count: ' + context.raw + ')';
                    }
                }
            },
        }
    });
});
</script>
